@@ -18,7 +18,7 @@
 # those files. Users are asked to read the 3rd Party Licenses
 # referenced with those assets.
 #
-# Copyright (c) 2021 Tom Kralidis
+# Copyright (c) 2022 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -44,15 +44,18 @@
 # =================================================================
 
 import json
+import logging
 import os
 from typing import Union
 
-from pygeometa.core import get_charstring
+from pygeometa.core import get_charstring, normalize_datestring
 from pygeometa.helpers import json_serial
 from pygeometa.schemas.base import BaseOutputSchema
 
 THISDIR = os.path.dirname(os.path.realpath(__file__))
 
+LOGGER = logging.getLogger(__name__)
+
 
 class OGCAPIRecordOutputSchema(BaseOutputSchema):
     """OGC API - Records - Part 1: Core record schema"""
@@ -78,22 +81,25 @@ class OGCAPIRecordOutputSchema(BaseOutputSchema):
         :returns: `dict` or `str` of MCF as an OARec record representation
         """
 
-        lang1 = mcf['metadata'].get('language')
-        lang2 = mcf['metadata'].get('language_alternate')
+        self.lang1 = mcf['metadata'].get('language')
+        self.lang2 = mcf['metadata'].get('language_alternate')
 
         minx, miny, maxx, maxy = (mcf['identification']['extents']
                                   ['spatial'][0]['bbox'])
 
         title = get_charstring(mcf['identification'].get('title'),
-                               lang1, lang2)
+                               self.lang1, self.lang2)
 
         description = get_charstring(mcf['identification'].get('abstract'),
-                                     lang1, lang2)
+                                     self.lang1, self.lang2)
 
+        LOGGER.debug('Generating baseline record')
         record = {
             'id': mcf['metadata']['identifier'],
+            'conformsTo': [
+                'http://www.opengis.net/spec/ogcapi-records-1/1.0/req/record-core',  # noqa
+            ],
             'type': 'Feature',
-            'bbox': [minx, miny, maxx, maxy],
             'geometry': {
                 'type': 'Polygon',
                 'coordinates': [[
@@ -106,70 +112,75 @@ class OGCAPIRecordOutputSchema(BaseOutputSchema):
             },
             'properties': {
                 'identifier': mcf['metadata']['identifier'],
-                'externalId': [{
-                    'scheme': 'default',
-                    'value': mcf['metadata']['identifier']
-                }],
                 'title': title[0],
                 'description': description[0],
                 'themes': [],
                 'providers': [],
-                'language': lang1,
+                'language': self.lang1,
                 'type': mcf['metadata']['hierarchylevel'],
-                'extent': {
-                    'spatial': {
-                        'bbox': [[minx, miny, maxx, maxy]],
-                        'crs': 'http://www.opengis.net/def/crs/OGC/1.3/CRS84'  # noqa
-                    }
-                },
             },
             'links': []
         }
 
+        LOGGER.debug('Checking for temporal')
         if 'temporal' in mcf['identification']['extents']:
             begin = mcf['identification']['extents']['temporal'][0]['begin']
             end = mcf['identification']['extents']['temporal'][0]['end']
 
-            if end == 'now':
+            if begin in ['now', 'None', None]:
+                begin = None
+            else:
+                begin = str(begin)
+
+            if end in ['now', 'None', None]:
                 end = None
+            else:
+                end = str(end)
 
-            temporal = {
-                'interval': [begin, end],
-                'trs': 'http://www.opengis.net/def/uom/ISO-8601/0/Gregorian'  # noqa
+            record['time'] = {
+                'interval': [begin, end]
             }
+
             if 'resolution' in  mcf['identification']['extents']['temporal'][0]:  # noqa
-                temporal['resolution'] =  mcf['identification']['extents']['temporal'][0]['resolution']  # noqa
-
-            record['properties']['extent']['temporal'] = temporal
+                record['time']['resolution'] =  mcf['identification']['extents']['temporal'][0]['resolution']  # noqa
 
+        LOGGER.debug('Checking for dates')
         if 'creation' in mcf['identification']['dates']:
-            record['properties']['created'] = mcf['identification']['dates']['creation']  # noqa
+            record['properties']['created'] = str(mcf['identification']['dates']['creation'])  # noqa
         if 'revision' in mcf['identification']['dates']:
-            record['properties']['updated'] = mcf['identification']['dates']['revision']  # noqa
+            record['properties']['updated'] = str(mcf['identification']['dates']['revision'])  # noqa
+
+        if 'publication' in mcf['identification']['dates']:
+            record['properties']['recordUpdated'] = normalize_datestring(mcf['identification']['dates']['publication'])  # noqa
 
         rights = get_charstring(mcf['identification'].get('rights'),
-                                lang1, lang2)
+                                self.lang1, self.lang2)
 
         record['properties']['rights'] = rights[0]
 
         formats = []
         for v in mcf['distribution'].values():
-            format_ = get_charstring(v.get('format'), lang1, lang2)
+            format_ = get_charstring(v.get('format'), self.lang1, self.lang2)
             if format_[0] is not None:
                 formats.append(format_)
 
+        LOGGER.debug('Checking for formats')
         if formats:
             record['properties']['formats'] = list(
                 set([f[0] for f in formats]))
 
+        LOGGER.debug('Checking for contacts')
         for key, value in mcf['contact'].items():
             record['properties']['providers'].append(
-                self.generate_responsible_party(value, lang1, lang2, key))
+                self.generate_responsible_party(value, self.lang1,
+                                                self.lang2, key))
 
+        LOGGER.debug('Checking for keywords')
         for value in mcf['identification']['keywords'].values():
             theme = {'concepts': []}
 
-            keywords = get_charstring(value.get('keywords'), lang1, lang2)
+            keywords = get_charstring(value.get('keywords'), self.lang1,
+                                      self.lang2)
 
             for kw in keywords[0]:
                 theme['concepts'].append(kw)
@@ -182,27 +193,9 @@ class OGCAPIRecordOutputSchema(BaseOutputSchema):
 
             record['properties']['themes'].append(theme)
 
+        LOGGER.debug('Checking for distribution')
         for value in mcf['distribution'].values():
-            title = get_charstring(value.get('title'), lang1, lang2)
-
-            name = get_charstring(value.get('name'), lang1, lang2)
-
-            reltype = value.get('links') or value.get('function')
-
-            link = {
-                'rel': reltype,
-                'href': value['url'],
-                'type': value['type']
-            }
-            if title != [None, None]:
-                link['title'] = name[0]
-            elif name != [None, None]:
-                link['title'] = name[0]
-
-            if all(x in value['url'] for x in ['{', '}']):
-                link['templated'] = True
-
-            record['links'].append(link)
+            record['links'].append(self.generate_link(value))
 
         if stringify:
             return json.dumps(record, default=json_serial, indent=4)
@@ -215,31 +208,38 @@ class OGCAPIRecordOutputSchema(BaseOutputSchema):
         generate responsibly party construct from MCF contact
 
         :param contact: dict of MCF contact
-        :param lang1: primary language
-        :param lang2: alternate language
+        :param self.lang1: primary language
+        :param self.lang2: alternate language
         :param role: role of contact
 
         :returns: MCF contact as a responsible party representation
         """
 
         organization_name = get_charstring(contact.get('organization'),
-                                           lang1, lang2)
+                                           self.lang1, self.lang2)
 
         position_name = get_charstring(contact.get('positionname'),
-                                       lang1, lang2)
+                                       self.lang1, self.lang2)
 
         hours_of_service = get_charstring(contact.get('hoursofservice'),
-                                          lang1, lang2)
+                                          self.lang1, self.lang2)
 
         contact_instructions = get_charstring(
-            contact.get('contactinstructions'), lang1, lang2)
+            contact.get('contactinstructions'), self.lang1, self.lang2)
+
+        address = get_charstring(contact.get('address'),
+                                 self.lang1, self.lang2)
+
+        city = get_charstring(contact.get('city'), self.lang1, self.lang2)
 
-        address = get_charstring(contact.get('address'), lang1, lang2)
-        city = get_charstring(contact.get('city'), lang1, lang2)
         administrative_area = get_charstring(contact.get('administrativearea'),
-                                             lang1, lang2)
-        postalcode = get_charstring(contact.get('postalcode'), lang1, lang2)
-        country = get_charstring(contact.get('country'), lang1, lang2)
+                                             self.lang1, self.lang2)
+
+        postalcode = get_charstring(contact.get('postalcode'),
+                                    self.lang1, self.lang2)
+
+        country = get_charstring(contact.get('country'),
+                                 self.lang1, self.lang2)
 
         rp = {
             'name': organization_name[0],
@@ -280,3 +280,34 @@ class OGCAPIRecordOutputSchema(BaseOutputSchema):
             }
 
         return rp
+
+    def generate_link(self, distribution: dict) -> dict:
+        """
+        Generates OARec link object from MCF distribution object
+
+        :param distribution: `dict` of MCF distribution
+
+        :returns: OARec link object
+        """
+
+        title = get_charstring(distribution.get('title'),
+                               self.lang1, self.lang2)
+
+        name = get_charstring(distribution.get('name'), self.lang1, self.lang2)
+
+        reltype = distribution.get('rel') or distribution.get('function')
+
+        link = {
+            'rel': reltype,
+            'href': distribution['url'],
+            'type': distribution['type']
+        }
+        if title != [None, None]:
+            link['title'] = name[0]
+        elif name != [None, None]:
+            link['title'] = name[0]
+
+        if all(x in distribution['url'] for x in ['{', '}']):
+            link['templated'] = True
+
+        return link
